@@ -1,32 +1,35 @@
-
 shared_params_save <- function(file_output=TRUE, warn_unsupported_types=FALSE) {
   c('@param object The object to save.',
     '@param file The file name/path.'[file_output],
-    '@param compress_level The compression level used (default 3).',
+    '@param compress_level The compression level used (the initial value is 3L).',
     '',
-    'The maximum and minimum possible values depends on the version of ZSTD library used.',
-    'As of ZSTD 1.5.6 the maximum compression level is 22, and the minimum is -131072. Usually, values in the low positive range offer very good performance in terms',
+    'The maximum and minimum possible values depend on the version of the ZSTD library used.',
+    'As of ZSTD 1.5.6 the maximum compression level is 22, and the minimum is -131072.',
+    'Usually, values in the low positive range offer very good performance in terms',
     'of speed and compression.',
-    '@param shuffle Whether to allow byte shuffling when compressing data (default: `TRUE`).',
-    '@param warn_unsupported_types Whether to warn when saving an object with an unsupported type (default `TRUE`).'[warn_unsupported_types],
-    '@param nthreads The number of threads to use when compressing data (default: `1`).'
+    '@param shuffle Whether to allow byte shuffling when compressing data (the initial value is TRUE).',
+    '@param warn_unsupported_types Whether to warn when saving an object with an unsupported type (the initial value is TRUE).'[warn_unsupported_types],
+    '@param nthreads The number of threads to use when compressing data (the initial value is 1L).'
     )
 }
 
 shared_params_read <- function(file_input=TRUE, use_alt_rep=FALSE) {
   c('@param file The file name/path.'[file_input],
-   '@param input The raw vector to deserialize.'[!file_input],
-   '@param use_alt_rep Use ALTREP when reading in string data (default `FALSE`).'[use_alt_rep],
-   '@param validate_checksum Whether to validate the stored checksum in the file (default `FALSE`). This can be used to test for file corruption but has a performance penality.',
-   '@param nthreads The number of threads to use when reading data (default: `1`).')
+    '@param input The raw vector to deserialize.'[!file_input],
+    '@param use_alt_rep Use ALTREP when reading in string data (the initial value is FALSE).'[use_alt_rep],
+    '@param validate_checksum Whether to validate the stored checksum in the file (the initial value is FALSE).',
+    '@param nthreads The number of threads to use when reading data (the initial value is 1L).'
+  )
 }
 
 #' qs_save
 #'
 #' Saves an object to disk using the `qs2` format.
 #'
-#' @usage qs_save(object, file, compress_level = 3L,
-#' shuffle = TRUE, nthreads = 1L)
+#' @usage qs_save(object, file,
+#'          compress_level = qopt("compress_level"),
+#'          shuffle = qopt("shuffle"),
+#'          nthreads = qopt("nthreads"))
 #'
 #' @eval shared_params_save()
 #' @return No value is returned. The file is written to disk.
@@ -35,20 +38,23 @@ shared_params_read <- function(file_input=TRUE, use_alt_rep=FALSE) {
 #'
 #' @examples
 #' x <- data.frame(int = sample(1e3, replace=TRUE),
-#'         num = rnorm(1e3),
-#'         char = sample(state.name, 1e3, replace=TRUE),
-#'         stringsAsFactors = FALSE)
+#'          num = rnorm(1e3),
+#'          char = sample(state.name, 1e3, replace=TRUE),
+#'          stringsAsFactors = FALSE)
 #' myfile <- tempfile()
 #' qs_save(x, myfile)
 #' x2 <- qs_read(myfile)
-#' identical(x, x2) # returns true
+#' identical(x, x2) # returns TRUE
 NULL
 
 #' qs_serialize
 #' 
 #' Serializes an object to a raw vector using the `qs2` format.
 #' 
-#' @usage qs_serialize(object, compress_level = 3L, shuffle = TRUE, nthreads = 1L)
+#' @usage qs_serialize(object,
+#'          compress_level = qopt("compress_level"),
+#'          shuffle = qopt("shuffle"),
+#'          nthreads = qopt("nthreads"))
 #' 
 #' @eval shared_params_save(file_output=FALSE)
 #' @return The serialized object as a raw vector.
@@ -57,19 +63,21 @@ NULL
 #' 
 #' @examples
 #' x <- data.frame(int = sample(1e3, replace=TRUE),
-#'         num = rnorm(1e3),
-#'         char = sample(state.name, 1e3, replace=TRUE),
-#'         stringsAsFactors = FALSE)
+#'          num = rnorm(1e3),
+#'          char = sample(state.name, 1e3, replace=TRUE),
+#'          stringsAsFactors = FALSE)
 #' xserialized <- qs_serialize(x)
 #' x2 <- qs_deserialize(xserialized)
-#' identical(x, x2) # returns true
+#' identical(x, x2) # returns TRUE
 NULL
 
 #' qs_read
 #'
 #' Reads an object that was saved to disk in the `qs2` format.
 #'
-#' @usage qs_read(file, validate_checksum=FALSE, nthreads = 1L)
+#' @usage qs_read(file,
+#'          validate_checksum = qopt("validate_checksum"),
+#'          nthreads = qopt("nthreads"))
 #'
 #' @eval shared_params_read()
 #' @return The object stored in `file`.
@@ -78,20 +86,22 @@ NULL
 #'
 #' @examples
 #' x <- data.frame(int = sample(1e3, replace=TRUE),
-#'         num = rnorm(1e3),
-#'         char = sample(state.name, 1e3, replace=TRUE),
-#'         stringsAsFactors = FALSE)
+#'          num = rnorm(1e3),
+#'          char = sample(state.name, 1e3, replace=TRUE),
+#'          stringsAsFactors = FALSE)
 #' myfile <- tempfile()
 #' qs_save(x, myfile)
 #' x2 <- qs_read(myfile)
-#' identical(x, x2) # returns true
+#' identical(x, x2) # returns TRUE
 NULL
 
 #' qs_deserialize
 #' 
 #' Deserializes a raw vector to an object using the `qs2` format.
 #' 
-#' @usage qs_deserialize(input, validate_checksum = FALSE, nthreads = 1L)
+#' @usage qs_deserialize(input,
+#'          validate_checksum = qopt("validate_checksum"),
+#'          nthreads = qopt("nthreads"))
 #' 
 #' @eval shared_params_read(file_input=FALSE)
 #' @return The deserialized object.
@@ -100,21 +110,23 @@ NULL
 #' 
 #' @examples
 #' x <- data.frame(int = sample(1e3, replace=TRUE),
-#'         num = rnorm(1e3),
-#'         char = sample(state.name, 1e3, replace=TRUE),
-#'         stringsAsFactors = FALSE)
+#'          num = rnorm(1e3),
+#'          char = sample(state.name, 1e3, replace=TRUE),
+#'          stringsAsFactors = FALSE)
 #' xserialized <- qs_serialize(x)
 #' x2 <- qs_deserialize(xserialized)
-#' identical(x, x2) # returns true
+#' identical(x, x2) # returns TRUE
 NULL
 
 #' qd_save
 #'
 #' Saves an object to disk using the `qdata` format.
 #'
-#' @usage qd_save(object, file, compress_level = 3L,
-#' shuffle = TRUE, warn_unsupported_types=TRUE,
-#' nthreads = 1L)
+#' @usage qd_save(object, file,
+#'          compress_level = qopt("compress_level"),
+#'          shuffle = qopt("shuffle"),
+#'          warn_unsupported_types = qopt("warn_unsupported_types"),
+#'          nthreads = qopt("nthreads"))
 #'
 #' @eval shared_params_save(warn_unsupported_types = TRUE)
 #' @return No value is returned. The file is written to disk.
@@ -123,21 +135,24 @@ NULL
 #'
 #' @examples
 #' x <- data.frame(int = sample(1e3, replace=TRUE),
-#'         num = rnorm(1e3),
-#'         char = sample(state.name, 1e3, replace=TRUE),
-#'         stringsAsFactors = FALSE)
+#'          num = rnorm(1e3),
+#'          char = sample(state.name, 1e3, replace=TRUE),
+#'          stringsAsFactors = FALSE)
 #' myfile <- tempfile()
 #' qd_save(x, myfile)
 #' x2 <- qd_read(myfile)
-#' identical(x, x2) # returns true
+#' identical(x, x2) # returns TRUE
 NULL
 
 #' qd_serialize
 #' 
 #' Serializes an object to a raw vector using the `qdata` format.
 #' 
-#' @usage qd_serialize(object, compress_level = 3L, shuffle = TRUE, 
-#' warn_unsupported_types = TRUE, nthreads = 1L)
+#' @usage qd_serialize(object,
+#'          compress_level = qopt("compress_level"),
+#'          shuffle = qopt("shuffle"),
+#'          warn_unsupported_types = qopt("warn_unsupported_types"),
+#'          nthreads = qopt("nthreads"))
 #' 
 #' @eval shared_params_save(file_output=FALSE, warn_unsupported_types = TRUE)
 #' @return The serialized object as a raw vector.
@@ -146,19 +161,22 @@ NULL
 #' 
 #' @examples
 #' x <- data.frame(int = sample(1e3, replace=TRUE),
-#'         num = rnorm(1e3),
-#'         char = sample(state.name, 1e3, replace=TRUE),
-#'         stringsAsFactors = FALSE)
+#'          num = rnorm(1e3),
+#'          char = sample(state.name, 1e3, replace=TRUE),
+#'          stringsAsFactors = FALSE)
 #' xserialized <- qd_serialize(x)
 #' x2 <- qd_deserialize(xserialized)
-#' identical(x, x2) # returns true
+#' identical(x, x2) # returns TRUE
 NULL
 
 #' qd_read
 #'
 #' Reads an object that was saved to disk in the `qdata` format.
 #'
-#' @usage qd_read(file, use_alt_rep = FALSE, validate_checksum=FALSE, nthreads = 1L)
+#' @usage qd_read(file,
+#'          use_alt_rep = qopt("use_alt_rep"),
+#'          validate_checksum = qopt("validate_checksum"),
+#'          nthreads = qopt("nthreads"))
 #'
 #' @eval shared_params_read(use_alt_rep = TRUE)
 #' @return The object stored in `file`.
@@ -167,20 +185,23 @@ NULL
 #'
 #' @examples
 #' x <- data.frame(int = sample(1e3, replace=TRUE),
-#'         num = rnorm(1e3),
-#'         char = sample(state.name, 1e3, replace=TRUE),
-#'         stringsAsFactors = FALSE)
+#'          num = rnorm(1e3),
+#'          char = sample(state.name, 1e3, replace=TRUE),
+#'          stringsAsFactors = FALSE)
 #' myfile <- tempfile()
 #' qd_save(x, myfile)
 #' x2 <- qd_read(myfile)
-#' identical(x, x2) # returns true
+#' identical(x, x2) # returns TRUE
 NULL
 
 #' qd_deserialize
 #' 
 #' Deserializes a raw vector to an object using the `qdata` format.
 #' 
-#' @usage qd_deserialize(input, use_alt_rep = FALSE, validate_checksum = FALSE, nthreads = 1L)
+#' @usage qd_deserialize(input,
+#'          use_alt_rep = qopt("use_alt_rep"),
+#'          validate_checksum = qopt("validate_checksum"),
+#'          nthreads = qopt("nthreads"))
 #' 
 #' @eval shared_params_read(file_input=FALSE, use_alt_rep = TRUE)
 #' @return The deserialized object.
@@ -189,12 +210,12 @@ NULL
 #' 
 #' @examples
 #' x <- data.frame(int = sample(1e3, replace=TRUE),
-#'         num = rnorm(1e3),
-#'         char = sample(state.name, 1e3, replace=TRUE),
-#'         stringsAsFactors = FALSE)
+#'          num = rnorm(1e3),
+#'          char = sample(state.name, 1e3, replace=TRUE),
+#'          stringsAsFactors = FALSE)
 #' xserialized <- qd_serialize(x)
 #' x2 <- qd_deserialize(xserialized)
-#' identical(x, x2) # returns true
+#' identical(x, x2) # returns TRUE
 NULL
 
 #' qx_dump
@@ -212,9 +233,9 @@ NULL
 #'
 #' @examples
 #' x <- data.frame(int = sample(1e3, replace=TRUE),
-#'         num = rnorm(1e3),
-#'         char = sample(state.name, 1e3, replace=TRUE),
-#'         stringsAsFactors = FALSE)
+#'          num = rnorm(1e3),
+#'          char = sample(state.name, 1e3, replace=TRUE),
+#'          stringsAsFactors = FALSE)
 #' myfile <- tempfile()
 #' qs_save(x, myfile)
 #' binary_data <- qx_dump(myfile)
@@ -224,7 +245,7 @@ NULL
 #'
 #' Compresses to a raw vector using the zstd algorithm. Exports the main zstd compression function.
 #'
-#' @usage zstd_compress_raw(data, compress_level)
+#' @usage zstd_compress_raw(data, compress_level = qopt("compress_level"))
 #'
 #' @param data Raw vector to be compressed.
 #' @param compress_level The compression level used.
@@ -267,7 +288,7 @@ NULL
 #'
 #' @param size An integer size
 #'
-#' @return maximum compressed size
+#' @return Maximum compressed size.
 #' @export
 #' @name zstd_compress_bound
 #'
@@ -276,7 +297,6 @@ NULL
 #' zstd_compress_bound(1e9)
 NULL
 
-
 #' Shuffle a raw vector
 #'
 #' Shuffles a raw vector using BLOSC shuffle routines.
@@ -286,7 +306,7 @@ NULL
 #' @param data A raw vector to be shuffled.
 #' @param bytesofsize Either `4` or `8`.
 #'
-#' @return The shuffled vector
+#' @return The shuffled vector.
 #' @export
 #' @name blosc_shuffle_raw
 #'
@@ -317,12 +337,12 @@ NULL
 
 #' XXH3_64 hash
 #' 
-#' Calculates 64-bit XXH3 hash
+#' Calculates a 64-bit XXH3 hash.
 #' 
 #' @usage xxhash_raw(data)
-#' @param data The data to hash
+#' @param data The data to hash.
 #' 
-#' @return The 64-bit hash
+#' @return The 64-bit hash.
 #' @export
 #' @name xxhash_raw
 #' 
@@ -330,4 +350,3 @@ NULL
 #' x <- as.raw(c(1,2,3))
 #' xxhash_raw(x)
 NULL
-
